@@ -42,7 +42,7 @@
  * @author Akinobu LEE
  * @date   Tue Feb 22 17:00:45 2005
  *
- * $Revision: 1.11 $
+ * $Revision: 1.14 $
  * 
  */
 /*
@@ -1256,7 +1256,7 @@ node_exist_token(FSBeam *d, int tt, int node, WORD_ID wid)
     }
 #ifdef WPAIR_KEEP_NLIMIT
     if (lowest_token == TOKENID_UNDEFINED ||
-	d->tlist[tt][lowest_token].score < d->tlist[tt][tmp].score)
+	d->tlist[tt][lowest_token].score > d->tlist[tt][tmp].score)
       lowest_token = tmp;
     if (++i >= d->wpair_keep_nlimit) break;
 #endif
@@ -1703,11 +1703,15 @@ init_nodescore(HTK_Param *param, RecogProcess *r)
 	      newid = create_token(d);
 	      new = &(d->tlist[d->tn][newid]);
 	      new->last_tre = &(d->bos);
+#ifdef FIX_PENALTY
 	      new->last_lscore = 0.0;
+#else
+	      new->last_lscore = d->penalty1;
+#endif
 	      if (wchmm->hmminfo->multipath) {
-		new->score = 0.0;
+		new->score = new->last_lscore;
 	      } else {
-		new->score = outprob_style(wchmm, node, d->bos.wid, 0, param);
+		new->score = outprob_style(wchmm, node, d->bos.wid, 0, param) + new->last_lscore;
 	      }
 	      node_assign_token(d, node, newid);
 	    }
@@ -2180,11 +2184,7 @@ save_trellis(BACKTRELLIS *bt, WCHMM_INFO *wchmm, TOKEN2 *tk, int t, boolean fina
   tre->begintime = tk->last_tre->endtime + 1; /* word beginning frame */
   tre->endtime   = t-1;	/* word end frame */
   tre->last_tre  = tk->last_tre; /* link to previous trellis word */
-  if (wchmm->lmtype == LM_PROB) {
-    tre->lscore    = tk->last_lscore;	/* log score (LM only) */
-  } else if (wchmm->lmtype == LM_DFA) {
-    tre->lscore = 0.0;
-  }
+  tre->lscore    = tk->last_lscore;	/* log LM score  */
   bt_store(bt, tre); /* save to backtrellis */
 #ifdef WORD_GRAPH
   if (tre->last_tre != NULL) {
@@ -2252,6 +2252,7 @@ beam_inter_word(WCHMM_INFO *wchmm, FSBeam *d, TOKEN2 **tk_ret, TRELLIS_ATOM *tre
     if (sword == wchmm->winfo->tail_silwid) return;
 
 #ifdef UNIGRAM_FACTORING
+#ifndef WPAIR
     /* ȤǶͭñƬΡɤФñܤޤȤƷ׻뤿ᡤ*/
     /* Υ롼ǤϺ٤ñ콪üΡɤϿƤ */
     /* here we will record the best wordend node of maximum likelihood
@@ -2265,6 +2266,7 @@ beam_inter_word(WCHMM_INFO *wchmm, FSBeam *d, TOKEN2 **tk_ret, TRELLIS_ATOM *tre
       d->wordend_best_tre = tre;
       d->wordend_best_last_cword = tk->last_cword;
     }
+#endif
 #endif
     
     /* N-gramˤƤϾñ³θɬפ뤿ᡤ
@@ -2302,6 +2304,23 @@ beam_inter_word(WCHMM_INFO *wchmm, FSBeam *d, TOKEN2 **tk_ret, TRELLIS_ATOM *tre
       /* N-gramΨ׻ */
       /* compute N-gram probability */
 #ifdef UNIGRAM_FACTORING
+      /* wchmm,start2isolate[0..stid-1] ... Ρɤͭʤñ
+	 ̤ID, ͭ(åɬפΤʤ)ñ -1 */
+      /* wchmm->start2isolate[0..stid-1] ... isolate ID for
+	 beginning-of-word state.  value: -1 for states that has
+	 1-gram factoring value (share nodes with some other words),
+	 and ID for unshared words
+      */
+      isoid = wchmm->start2isolate[stid];
+#ifdef WPAIR
+      /* Efficient cross-word LM handling should be disabled for
+	 word-pair approximation */
+      if (isoid == -1) {
+	tmpprob = wchmm->fscore[- wchmm->state[next_node].scid];
+      } else {
+	tmpprob = iwparray[isoid];
+      }
+#else  /* ~WPAIR */
       /* 1-gram factoring ˤñָΨåθΨ:
 	 1-gram factoring ñ˰¸ʤΤǡ
 	 ǻȤ factoring ͤ¿
@@ -2316,21 +2335,14 @@ beam_inter_word(WCHMM_INFO *wchmm, FSBeam *d, TOKEN2 **tk_ret, TRELLIS_ATOM *tre
 	 So only the unshared beginning-of-word states are computed and
 	 cached here in iwparray[].
       */
-      /* wchmm,start2isolate[0..stid-1] ... Ρɤͭʤñ
-	 ̤ID, ͭ(åɬפΤʤ)ñ -1 */
-      /* wchmm->start2isolate[0..stid-1] ... isolate ID for
-	 beginning-of-word state.  value: -1 for states that has
-	 1-gram factoring value (share nodes with some other words),
-	 and ID for unshared words
-      */
-      isoid = wchmm->start2isolate[stid];
       /* ׻ɬפǤʤñƬΡɤϥѥޤȤƸ˷׻Τ
 	 Ǥϥå */
       /* the shared nodes will be computed afterward, so just skip them
 	 here */
       if (isoid == -1) continue;
       tmpprob = iwparray[isoid];
-#else
+#endif /* ~WPAIR */
+#else  /* ~UNIGRAM_FACTORING */
       tmpprob = iwparray[stid];
 #endif
     }
@@ -2374,7 +2386,8 @@ beam_inter_word(WCHMM_INFO *wchmm, FSBeam *d, TOKEN2 **tk_ret, TRELLIS_ATOM *tre
     if (wchmm->lmtype == LM_DFA) {
       /* grammar: ñڥʥƥɲ */
       /* grammar: add insertion penalty */
-      tmpsum += d->penalty1;
+      ngram_score_cache = d->penalty1;
+      tmpsum += ngram_score_cache;
 
       /* grammar: deterministic factoring (in case category-tree not enabled) */
       if (!wchmm->category_tree) {
@@ -2621,6 +2634,7 @@ get_back_trellis_proceed(int t, HTK_Param *param, RecogProcess *r, boolean final
   tn = d->tn;
 
 #ifdef UNIGRAM_FACTORING
+#ifndef WPAIR
   /* 1-gram factoring ǤñƬǤθΨľñ˰¸ʤ
      ᡤñ Viterbi ˤФľñ,ñˤ餺̤Ǥ. 
      äñ콪üfactoringͤΤñƬؤܤϣĤˤޤȤ. 
@@ -2642,6 +2656,7 @@ get_back_trellis_proceed(int t, HTK_Param *param, RecogProcess *r, boolean final
     d->wordend_best_score = LOG_ZERO;
   }
 #endif
+#endif
 
 #ifdef DEBUG
   /* debug */
@@ -2797,6 +2812,7 @@ get_back_trellis_proceed(int t, HTK_Param *param, RecogProcess *r, boolean final
   }
 
 #ifdef UNIGRAM_FACTORING
+#ifndef WPAIR
 
   if (lmtype == LM_PROB) {
 
@@ -2809,6 +2825,7 @@ get_back_trellis_proceed(int t, HTK_Param *param, RecogProcess *r, boolean final
       beam_inter_word_factoring(wchmm, d);
     }
   }
+#endif
 #endif /* UNIGRAM_FACTORING */
 
   /***************************************/
